
      SUBROUTINE gtapln(Center, Delta, ktval, Emlim, qwave, retstr, 
     &                  status)

      IMPLICIT NONE

      REAL Center, Delta, Ktval, Emlim
      INTEGER status
      CHARACTER retstr*(*)
      LOGICAL qwave

c Routine to returns IDs of lines in the range Center+/-Delta. Each line has
c an 70 character descriptor which is concatenated into the returned string.
c This uses the APEC line file and returns lines for the tabulated temperature
c closest to that input.
c  Arguments :
c      Center        r        i: Center energy of search range (keV)
c      Delta         r        i: Half-width of search range (keV)
c      ktval         r        i: Temperature (keV)
c      qwave         l        i: If true then output wavelengths
c      retstr        c        r: Descriptors
c      status        i        r: 0==OK, -1==too many lines in range


      REAL keVtoA
      PARAMETER(keVtoA=12.398419)

      REAL wave, energy, emiss, ktdiff, ktread, output

      INTEGER ilun, icol(6), ion, iup, ilow, lentmp
      INTEGER ielt, index, ipt, nrow, i, lenstr, lenn

      LOGICAL qanyf, qexist

      CHARACTER*20 srcfil
      CHARACTER infile*255, datad*128, pname*128, pvalue*128
      CHARACTER contxt*255, comment*50, tmpstr*255
      CHARACTER*10 colnam(6)
      CHARACTER*2 celt(28)
      CHARACTER*5 cion(26)

      INTEGER lenact
      CHARACTER*128 fgdatd, fgmstr
      EXTERNAL fgdatd, lenact, fgmstr

      DATA srcfil / 'apec_v1.10_line.fits' /
      DATA colnam / 'Lambda', 'Epsilon', 'Element', 'Ion', 'UpperLev',
     &              'LowerLev' /
      DATA celt   / 5*'  ', 'C ', 'N ', 'O ', 'F ', 'Ne', 'Na', 'Mg',
     &              'Al', 'Si', 'P ', 'S ', 'Cl', 'Ar', 'K ', 'Ca',
     &              5*'  ', 'Fe', 'Co', 'Ni' /
      DATA cion   / 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII',
     &              'IX', 'X', 'XI', 'XII', 'XIII', 'XIV', 'XV', 'XVI',
     &              'XVII', 'XVIII', 'XIX', 'XX', 'XXI', 'XXII', 
     &              'XXIII', 'XXIV', 'XXV', 'XXVI' /


      status = 0
      retstr = ' '

c Open the file with the line energy and ID information and go to the table
c extension with temperatures.

      datad = fgdatd()
      lenn = lenact(datad)
      pname = 'APECROOT'
      pvalue = fgmstr(pname)
      lentmp = lenact(pvalue)

      IF ( pvalue .EQ. ' ' ) THEN
         infile = datad(:lenn)//srcfil
      ELSE

c if APECROOT has been set first test whether only a version number has
c changed. Then assume that only a root filename has been given. If neither
c these work then assume that an entire directory path was included.

         infile = datad(:lenn)//'apec_v'//
     &             pvalue(:lentmp)//'_line.fits'
         inquire(file=infile, exist=qexist)
         IF ( .NOT.qexist ) THEN
            infile = datad(:lenn)//pvalue(:lentmp)//'_line.fits'
            inquire(file=infile, exist=qexist)
            IF ( .NOT.qexist ) THEN
               infile = pvalue(:lentmp)//'_line.fits'
            ENDIF
         ENDIF
      ENDIF

      contxt = 'Reading line data from '//infile(:lenact(infile))
      CALL xwrite(contxt, 15)

      CALL getlun(ilun)
      CALL xsftop(ilun, infile, 0, index, status)
      contxt = 'Failed to open file '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

      CALL ftmahd(ilun, 2, index, status)
      contxt = 'Failed go to extension in '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

c Get the column we need

      CALL ftgcno(ilun, .FALSE., 'kt', icol(1), status)
      contxt = 'Failed get columns in '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

      CALL ftgkyj(ilun, 'NAXIS2', nrow, comment, status)
      contxt = 'Failed get NAXIS2 in '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

c Read the temperatures finding the one closest to the input

      ktdiff = 1.e30
      ipt = 0

      DO i = 1, nrow

         CALL ftgcve(ilun, icol(1), i, 1, 1, 0., ktread, qanyf, status)
         contxt = 'Failed to read temperatures'
         IF ( status .NE. 0 ) GOTO 999

         IF ( ktdiff .GT. ABS(ktval - ktread) ) THEN
            ktdiff = ABS(ktval - ktread)
            ipt = i
         ENDIF

      ENDDO

c Go to the indicated extension

      CALL ftmahd(ilun, 2+ipt, index, status)
      contxt = 'Failed go to line extension in '//
     &          infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

c Get the columns we need

      DO i = 1, 6
         CALL ftgcno(ilun, .FALSE., colnam(i), icol(i), status)
      ENDDO
      contxt = 'Failed get columns in '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

      CALL ftgkyj(ilun, 'NAXIS2', nrow, comment, status)
      contxt = 'Failed get NAXIS2 line list extension'
      IF ( status .NE. 0 ) GOTO 999

c Run through the list finding lines with energies in the specified
c range. Truncate the list if we run out of space in the string.

      ipt = 0
      lenstr = len(retstr)

      DO i = 1, nrow

         CALL ftgcve(ilun, icol(1), i, 1, 1, 0., wave, qanyf, status)
         contxt = 'Failed to read wavelength data'
         IF ( status .NE. 0 ) GOTO 999

         IF ( qwave ) THEN
            output = wave
         ELSE
            output = keVtoA/wave
         ENDIF

         IF ( output .GE. Center-Delta .AND. 
     &        output .LE. Center+Delta ) THEN

c inside the energy range of interest so read the rest of the information

            CALL ftgcve(ilun, icol(2), i, 1, 1, 0., emiss, qanyf, 
     &                  status)
            contxt = 'Failed to read flux data'
            IF ( status .NE. 0 ) GOTO 999

            CALL ftgcvj(ilun, icol(3), i, 1, 1, 0, ielt, qanyf, 
     &                  status)
            contxt = 'Failed to read element data'
            IF ( status .NE. 0 ) GOTO 999

            CALL ftgcvj(ilun, icol(4), i, 1, 1, 0, ion, qanyf, 
     &                  status)
            contxt = 'Failed to read ion data'
            IF ( status .NE. 0 ) GOTO 999

            CALL ftgcvj(ilun, icol(5), i, 1, 1, 0, iup, qanyf, 
     &                  status)
            contxt = 'Failed to read upper level data'
            IF ( status .NE. 0 ) GOTO 999

            CALL ftgcvj(ilun, icol(6), i, 1, 1, 0, ilow, qanyf, 
     &                  status)
            contxt = 'Failed to read lower level data'
            IF ( status .NE. 0 ) GOTO 999

c Write the string
            IF ( emiss .GE. emlim ) THEN
               IF ( ipt+70 .LE. lenstr ) THEN
                  WRITE(retstr(ipt+1:ipt+70), 
     & '(a22, f8.4, 5x, a2, 1x, a5, 1x, i5, 1x, i4, 5x, 1pe10.3)') 
     &               '     APEC           : ', output, celt(ielt), 
     &               cion(ion), iup, ilow, emiss
                  ipt = ipt + 70
               ELSE
                  CALL xwrite(
     &             'Too many lines - list will be truncated', 5)
                  CALL xwrite(
     & 'Try a higher cut-off emissivity or a smaller energy range', 5)
                  GOTO 999
               ENDIF
            ENDIF

         ENDIF

      ENDDO

 999  CONTINUE
      IF ( status .GT. 0 ) THEN
         CALL xwrite(contxt, 5)
         WRITE(contxt, '(a, i4)') 'FITSIO error = ', status
         CALL xwrite(contxt, 5)
         status = 0
      ENDIF

c Close the file

 1999 CONTINUE
      CALL ftclos(ilun, status)
      CALL frelun(ilun)

      RETURN
      END



