
      SUBROUTINE gtlnid(Center, Delta, source, qwave, retstr, status)

      IMPLICIT NONE

      REAL Center, Delta
      INTEGER status, source
      CHARACTER retstr*(*)
      LOGICAL qwave

c Routine to returns IDs of lines in the range Center+/-Delta. Each line has
c an 70 character descriptor which is concatenated into the returned string.
c  Arguments :
c      Center        r        i: Center energy of search range (keV)
c      Delta         r        i: Half-width of search range (keV)
c      source        i        i: Source of the IDs : 1==Bearden
c                                                    2==Mewe-Kaastra
c      qwave         l        i: If true then output wavelengths
c      retstr        c        r: Descriptors
c      status        i        r: 0==OK, -1==too many lines in range


      REAL keVtoA
      PARAMETER(keVtoA=12.398419)

      REAL wave, energy, fvalue

      INTEGER ilun, iline, icol(4)
      INTEGER ielt, index, ipt

      LOGICAL qdone, qanyf

      CHARACTER*20 srcfil(2)
      CHARACTER infile*255, datad*128, celt*2, cline*20
      CHARACTER cion*8, ctrans*8, contxt*255

      INTEGER lenact
      CHARACTER*128 fgdatd
      EXTERNAL fgdatd, lenact

      DATA srcfil / 'Bearden.fits', 'mekal_lines.fits' /

      retstr = ' '
      status = 0
      ipt = 0

c Open the file with the line energy and ID information and go to the table
c extension

      datad = fgdatd()
      infile = datad(:lenact(datad))//srcfil(source)
      CALL getlun(ilun)
      CALL xsftop(ilun, infile, 0, index, status)
      contxt = 'Failed to open file '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

      CALL ftmahd(ilun, 2, index, status)
      contxt = 'Failed go to extension in '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

c Get the columns we need

      IF ( source .EQ. 1 ) THEN
         CALL ftgcno(ilun, .FALSE., 'ENERGY', icol(1), status)
         CALL ftgcno(ilun, .FALSE., 'ELEMENT', icol(2), status)
         CALL ftgcno(ilun, .FALSE., 'TRANS', icol(3), status)
      ELSEIF (source .EQ. 2 ) THEN
         CALL ftgcno(ilun, .FALSE., 'WAVE', icol(1), status)
         CALL ftgcno(ilun, .FALSE., 'ION', icol(2), status)
         CALL ftgcno(ilun, .FALSE., 'TRANS', icol(3), status)
         CALL ftgcno(ilun, .FALSE., 'FVALUE', icol(4), status)
      ENDIF
      contxt = 'Failed get columns in '//infile(:lenact(infile))
      IF ( status .NE. 0 ) GOTO 999

c Run down the file till we get to Center+Delta or fall off the end of
c the file

      qdone = .FALSE.
      iline = 0
      DO WHILE ( .NOT.qdone )

         iline = iline + 1
         IF ( source .EQ. 1 ) THEN
            CALL ftgcve(ilun, icol(1), iline, 1, 1, 0., energy, qanyf, 
     &                  status)
            wave = keVtoA/energy
         ELSEIF ( source .EQ. 2 ) THEN
            CALL ftgcve(ilun, icol(1), iline, 1, 1, 0., wave, qanyf, 
     &                  status)
            energy = keVtoA/wave
         ENDIF
         IF ( status .NE. 0 ) qdone = .TRUE.
         IF ( qwave ) THEN
            IF ( wave .GE. Center-Delta ) qdone = .TRUE.
         ElSE
            IF ( energy .LE. Center+Delta ) qdone = .TRUE.
         ENDIF

      ENDDO

c If we fell off the bottom of the file then there were no lines found

      IF ( status .NE. 0 ) THEN
         status = 0
         GOTO 1999
      ENDIF

c Now write out all lines until the energy drops below Center-Delta or the
c file ends

      qdone = .FALSE.
      DO WHILE ( .NOT.qdone )

         IF ( ipt+70 .GT. len(retstr) ) THEN
            status = -1
            GOTO 999
         ENDIF

         IF ( source .EQ. 1 ) THEN
            CALL ftgcvs(ilun, icol(2), iline, 1, 1, ' ', celt, qanyf, 
     &                  status)
            CALL ftgcvs(ilun, icol(3), iline, 1, 1, ' ', cline, 
     &                  qanyf, status)
            IF ( qwave ) THEN
               WRITE(retstr(ipt+1:ipt+70), 
     &            '(5x, a15, 2x, f8.4, 5x, a2, 1x, a20, 12x)') 
     &            'Bearden      : ', (keVtoA/energy), celt, cline
            ELSE
               WRITE(retstr(ipt+1:ipt+70), 
     &            '(5x, a15, 2x, f8.4, 5x, a2, 1x, a20, 12x)') 
     &            'Bearden      : ', energy, celt, cline
            ENDIF
         ELSEIF ( source .EQ. 2 ) THEN
            CALL ftgcvs(ilun, icol(2), iline, 1, 1, ' ', cion, qanyf, 
     &                  status)
            CALL ftgcvs(ilun, icol(3), iline, 1, 1, ' ', ctrans,  
     &                  qanyf, status)
            CALL ftgcve(ilun, icol(4), iline, 1, 1, 0.0, fvalue,  
     &                  qanyf, status)
            CALL locase(cion(2:2))
            IF ( qwave ) THEN
               WRITE(retstr(ipt+1:ipt+70), 
     &            '(5x, a15, 2x, f8.4, 5x, a8, 1x, a8, 5x, f8.4, 5x)') 
     &            'Mewe-Kaastra : ', (keVtoA/energy), cion, ctrans, 
     &            fvalue
            ELSE
               WRITE(retstr(ipt+1:ipt+70), 
     &            '(5x, a15, 2x, f8.4, 5x, a8, 1x, a8, 5x, f8.4, 5x)') 
     &            'Mewe-Kaastra : ', energy, cion, ctrans, fvalue
            ENDIF
         ENDIF
         ipt = ipt + 70

         iline = iline + 1
         IF ( source .EQ. 1 ) THEN
            CALL ftgcve(ilun, icol(1), iline, 1, 1, 0., energy, 
     &                  qanyf, status)
            wave = keVtoA/energy
         ELSEIF ( source .EQ. 2 ) THEN
            CALL ftgcve(ilun, icol(1), iline, 1, 1, 0., wave, qanyf, 
     &                  status)
            energy = keVtoA/wave
         ENDIF

         IF ( status .EQ. 0 ) THEN
            IF ( qwave ) THEN
               IF ( wave .GT. Center+Delta ) qdone = .TRUE.
            ELSE
               IF ( energy .LT. Center-Delta ) qdone = .TRUE.
            ENDIF
         ELSE
            qdone = .TRUE.
         ENDIF

      ENDDO

 999  CONTINUE
      IF ( status .GT. 0 ) THEN
         CALL xwrite(contxt, 5)
         WRITE(contxt, '(a, i4)') 'FITSIO error = ', status
         CALL xwrite(contxt, 5)
         status = 0
      ENDIF

c Close the file

 1999 CONTINUE
      CALL ftclos(ilun, status)
      CALL frelun(ilun)

      RETURN
      END



