
      SUBROUTINE idntfy(Instrg, Lenn)

      IMPLICIT NONE

      INTEGER Lenn
      CHARACTER Instrg*(*)


c Subroutine to generate a list of possible identifications for the line
c energy input. The user gives a line energy and a delta energy in which
c to find lines. An optional redshift can be given. The line list to use
c can be specified.
c  Arguments :
c      Instrg     c          i: Parse string
c      Lenn       i        i/r: Parse position

      INTEGER MAXARG, MAXOPT
      REAL keVtoA
      PARAMETER(MAXARG=3, MAXOPT=3, keVtoA=12.398419)

      REAL argval(3), ktval, emlim
      REAL center, delta, Zfact

      INTEGER nret, iflag, idelim, ilun, status, i, j, iopt, jopt
      INTEGER lenb, lene

      CHARACTER*7000 retstr
      CHARACTER*72 wrtstr
      CHARACTER*20 argstr(3)
      CHARACTER*20 linlst, options(MAXOPT)

      LOGICAL qskip, qwave

      INTEGER lenact
      CHARACTER xpgxop*4
      EXTERNAL lenact, xpgxop

      SAVE argval, jopt

      DATA argval /1.0, 1.e-2, 0./
      DATA argstr /'Line energy (keV)', 'Delta energy (keV)', 
     &             'Redshift'/
      DATA options /'bearden', 'mekal', 'apec'/
      DATA jopt, ktval, emlim /3, 1., 1.e-19/

c If the user has setplot wave then use wavelengths instead of energies.

      qwave = .FALSE.
      IF ( xpgxop() .EQ. 'wave' ) qwave = .TRUE.

c Parse the input string to get the values

      idelim = 0
      CALL xgtrl4(Instrg, Lenn, 3, argstr, 3, argval, nret, iflag, 
     &            idelim)
      IF ( iflag .LT. 0 ) RETURN

      Zfact = 1. + argval(3)
      IF ( qwave ) THEN
         center = argval(1) / Zfact
         delta = argval(2) / Zfact
      ELSE
         center = argval(1) * Zfact
         delta = argval(2) * Zfact
      ENDIF

c and the line list to use...

      CALL xgtarg(Instrg, Lenn, lenb, lene, qskip, iflag, idelim)
      IF ( qskip .OR. iflag .LT. 0 ) RETURN

      IF ( iflag .EQ. 0 ) THEN
         iopt = jopt
         CALL xmatch(instrg(lenb:lene),options,MAXOPT,iopt)
         IF ( iopt .LE. 0 .OR. iopt .GT. MAXOPT .OR. iflag .NE. 0) THEN
            wrtstr='Line list  options (currently '''//
     &             options(jopt)(:lenact(options(jopt)))//''')'
            CALL xunids(instrg(lenb:lene),options,MAXOPT,iopt,
     &                  wrtstr)
            RETURN
         ENDIF
         jopt = iopt

c if this is the APEC line list then have the option of choosing a temperature
c and emissivity limit

         CALL xgtrl4(Instrg, Lenn, 1, 'Temperature', 1, ktval, nret, 
     &               iflag, idelim)
         IF ( iflag .LT. 0 ) RETURN

         CALL xgtrl4(Instrg, Lenn, 1, 'Emissivity limit', 1, emlim, 
     &               nret, iflag, idelim)
         IF ( iflag .LT. 0 ) RETURN

      ENDIF

c Get the line IDs in one long string.

      IF ( jopt .LE. 2 ) THEN

         CALL gtlnid(center, delta, jopt, qwave, retstr, status)

      ELSEIF ( jopt .EQ. 3 ) THEN

         CALL gtapln(center, delta, ktval, emlim, qwave, retstr, 
     &               status)

      ENDIF

c write them out

      CALL xwrite(' ', 10)
      DO i = 1, (lenact(retstr)+1)/70
         CALL xwrite(retstr(1+(i-1)*70:i*70), 10)
      ENDDO

      IF ( status .EQ. -1 ) THEN
         CALL xwrite(' ', 10)
         CALL xwrite('** Too many lines... output truncated **', 10)
         CALL xwrite(' ', 10)
      ENDIF

      CALL xwrite(' ', 10)

      RETURN
      END




