
      SUBROUTINE gtplin(igroup, icsr, icsp, ibin, phamod, phaobs, 
     &                  cvrobs, areasc, phacor, phabkg, cvrbkg, data, 
     &                  fmod, error, chemin, chemax, icb, ice, nbinrt, 
     &                  ixopt, qcount)

      INCLUDE '../../inc/xspec.inc'

      INTEGER ibin(MAXFIL)
      INTEGER igroup, icsr, icsp, icb, ice, nbinrt, ixopt
      REAL phamod(*), phaobs(*), cvrobs(*), phacor(*), phabkg(*)
      REAL cvrbkg(*), areasc(*)
      REAL data, fmod, error, chemin, chemax
      LOGICAL qcount

C---
C XSPEC subroutine to handle the grouping of data for plotting
C purposes, including multiple files and multiple channels.
C---
C arguments
C IGROUP    I    Current group to be accumulated
C ICSR      I    Current first relative channel
C ICSP      I    Maximum relative channel
C IBIN      I    Current bin number
C PHAMOD    I    Model values
C PHAOBS    I    Observed count rates
C CVROBS    I    Calculated Variances on PHAOBS
C AREASC    I    AREASCAL values
C PHACOR    I    Corrections on PHAOBS
C PHABKG    I    Background count rates
C CVRBKG    I    Variances on PHABKG
C DATA        O  Returned rebinned observed counts
C FMOD        O  Returned rebinned model value
C ERROR       O  Size of observed count error bar
C CHEMIN      O  Returned low channel energy
C CHEMAX      O  Returned high channel high energy bound
C ICB         O  First good channel (relative)
C ICE         O  Last good channel
C NBINRT      O  Total number of bins in accumulation,
C			when = 0, then there are no more data in
C			the range icsr to icsp to be accumulated
C ixopt    I     = 1 -> return flux/chan
c                = 2 -> return flux/keV
c                = 3 -> return flux/A
C QCOUNT    I    If .TRUE. then return the flux and model in units of counts.
C                
C---
C 22 May 1984 - rashafer
C 2.0: Modified to return the energy channels range as well as counts
C	in terms of per keV if qeax is true.
C 3.0: Use DSETSPEC structure
C---

      REAL NO, keVtoA
      PARAMETER(NO=-1.2e-34,keVtoA=12.398419)

      REAL tchmaxl, tchmaxh, anorm, crtvar, tmp, crtsig
      REAL chminl, chminh, chmaxl, chmaxh
      REAL count, bcount, afact, asum

      INTEGER icend, ifile, nend, nbeg, nexpb
      INTEGER ic, crtnch, errort

      LOGICAL qdone, found, qback

      REAL xpgcrs, dgcrnm
      INTEGER dgndst, dgplgr, dgchan, dgchbn, cgncmp, xpgcrn, xpgert
      LOGICAL dgqnot, fgqcrf
      REAL dgtime, rgerng
      EXTERNAL dgndst, dgplgr, dgchan, dgtime, xpgcrn, xpgert
      EXTERNAL dgchbn, rgerng, dgqnot, cgncmp, xpgcrs, dgcrnm, fgqcrf

c Initialize output variables

      icb = 0
      ice = 0
      nbinrt = 0
      data = 0.
      fmod = 0.
      error = 0.

c Check if there is anything to do. If the first channel exceeds
c the end or there are no files in the plotting group then return.

      IF ( icsr .GT. icsp ) RETURN
      found = .FALSE.
      DO ifile = 1, dgndst()
         IF (dgplgr(ifile).EQ.igroup) found = .TRUE.
      ENDDO
      IF ( .NOT. found ) RETURN

      ic = icsr
      icend = icsp
      crtsig = xpgcrs(igroup)
      crtnch = xpgcrn(igroup)
      errort = xpgert(igroup)
      crtvar = sign(crtsig*crtsig, crtsig)
      nbeg = 0
      chminl = 0.
      chminh = 0.
      chmaxl = 0.
      chmaxh = 0.
      count = 0.
      bcount = 0.
      asum = 0.
      qback = .FALSE.

      qdone = .FALSE.
      DO WHILE (.NOT.qdone)

         nend = 0
         tchmaxl = 0.
         tchmaxh = 0.

         DO ifile = 1, dgndst()

            IF ( (dgplgr(ifile) .NE. igroup) .OR.
     &           (ic .GT. dgchan(ifile))          ) GOTO 100

c If this is a noticed channel then process it

            IF ( dgqnot(ifile, ic) ) THEN

               IF ( dgbtme(ifile) .GT. 0. ) qback = .TRUE.

c If this is the first noticed channel in the file then find
c the bin number otherwise just increment.

               IF ( ibin(ifile) .EQ. 0 ) THEN
                  ibin(ifile) = dgchbn(ifile, ic)
               ELSE
                  ibin(ifile) = ibin(ifile) + 1
               ENDIF

c If this is the first bin and the first file then set the start
c channel number for the returned data.

               IF ( icb .EQ. 0 ) THEN
                  icb = ic
                  icend = MIN(icend, icb+crtnch-1)
               ENDIF

c If this is the first channel then save the nominal start energy

               IF ( ic .EQ. icb ) THEN
                  nbeg = nbeg + 1
                  chminl = chminl + rgerng(ic, ifile, 'l')
                  chminh = chminh + rgerng(ic, ifile, 'h')
               ENDIF

               nend = nend + 1
               tchmaxl = tchmaxl + rgerng(ic, ifile, 'l')
               tchmaxh = tchmaxh + rgerng(ic, ifile, 'h')
               nbinrt = nbinrt + 1

c accumulate the data, error, and model

               afact = dgtime(ifile)*areasc(ibin(ifile)) 
               tmp = 1. 
               IF (qcount) tmp = afact
               data = data + ( phaobs(ibin(ifile))
     &                        -phabkg(ibin(ifile)) ) * tmp
               count = count + phaobs(ibin(ifile))*afact
               bcount = bcount + phabkg(ibin(ifile))*afact
               asum = asum + afact
               IF ( fgqcrf(ifile) ) THEN
                  data = data - dgcrnm(ifile)*phacor(ibin(ifile))*tmp
                  count = count - dgcrnm(ifile)*phacor(ibin(ifile))
     &                                         *afact
               ENDIF
               IF ( cgncmp() .GT. 0 ) THEN
                  fmod = fmod + phamod(ibin(ifile))*tmp
               ENDIF
               error = error + (cvrobs(ibin(ifile))+cvrbkg(ibin(ifile))
     &                         )*tmp*tmp

               ice = ic

            ENDIF
 100        CONTINUE

         ENDDO

C Calculate chemax for the last bin with a non-zero
C contribution to the accumlated sums.

         IF (nend.GT.0) THEN
            chmaxl = tchmaxl/float(nend)
            chmaxh = tchmaxh/float(nend)
         ENDIF
         qdone = ((ic.GE.icend) .OR.
     &           ((data.GT.0.) .AND.
     &            (data*data.GE.error*crtvar)) )
         ic = ic + 1
      ENDDO

c If no bins were accumulated then return

      IF ( nbinrt .EQ. 0 ) RETURN

c Set the channel min energies

      IF (nbeg.GT.0) THEN
         chminl = chminl/float(nbeg)
         chminh = chminh/float(nbeg)
      ENDIF

c Now set the min and max channel energies. This convoluted procedure is
c necessary to handle the cases of both channels increasing and decreasing
c with energy

      chemin = MIN(MIN(chminl, chminh), MIN(chmaxl,chmaxh))
      chemax = MAX(MAX(chminl, chminh), MAX(chmaxl,chmaxh))

c Set the error. If we are summing in quadrature simply convert from 
c variance to standard deviations.

      asum = asum/FLOAT(nbinrt)

      IF ( errort .EQ. 1 ) THEN

         error = SQRT(error)

c If we are using sqrt(N) errors then calculate these and add source and
c background in quadrature. If we are not in count space then convert back
c using mean cm^2*sec.

      ELSEIF ( errort .EQ. 2 ) THEN

         error = SQRT(count + bcount)
         IF (.NOT.qcount) error = error/asum

c If we are using Gehrels type 1 error then do that

      ELSEIF ( errort .EQ. 3 ) THEN

         error = (1+SQRT(count+0.75))**2
         IF ( qback ) THEN
            error = error + (1+SQRT(bcount+0.75))**2
         ENDIF
         error = SQRT(error)
         IF (.NOT.qcount) error = error/asum

c If we are using Gehrels type 2 error then do that

      ELSEIF ( errort .EQ. 4 ) THEN

         error = count - 0.25
         IF ( qback ) THEN
            error = error + bcount - 0.25
         ENDIF
         error = SQRT(error)
         IF (.NOT.qcount) error = error/asum

c Finally if we are using the arithmetic mean of the type 1 and type 2
c errors

      ELSEIF ( errort .EQ. 5 ) THEN

         error = ((1+SQRT(count+0.75)+SQRT(count-0.25))/2.)**2
         IF ( qback ) THEN
            error = error + ((1+SQRT(bcount+0.75)
     &                         +SQRT(bcount-0.25))/2.)**2
         ENDIF
         error = SQRT(error)
         IF (.NOT.qcount) error = error/asum

      ENDIF

c Set the scaling factor to get data, error, and model in the correct
c units (note that the energy and wavelength normalizations assume that
c the no. of square cm. is the same in each grouped file). 
c First channels

      IF ( ixopt. EQ. 1 ) THEN

         anorm = 1./float(nbinrt)

c else energy (/keV)

      ELSEIF ( ixopt .EQ. 2 ) THEN

         nexpb = (ice-icb+1)

         IF (chemin .EQ. chemax) THEN
            anorm = 1.
         ELSE
            anorm = float(nexpb)/(abs(chemax-chemin)*float(nbinrt))
         ENDIF

c else wavelength (/A)

      ELSEIF ( ixopt .EQ. 3 ) THEN

         nexpb = (ice-icb+1)

         IF (chemin .EQ. chemax .OR. 
     &       chemin .EQ. NO .OR. chemin .LE. 0. .OR.
     &       chemax .EQ. NO .OR. chemax .LE. 0. ) THEN
            anorm = 1.
         ELSE
            anorm = float(nexpb)/(abs((keVtoA/chemax)-(keVtoA/chemin))
     &                            *float(nbinrt))
         ENDIF

      ELSE

         RETURN

      ENDIF

      IF ( anorm .NE. 1. ) THEN
         error = error*anorm
         data = data*anorm
         fmod = fmod*anorm
      ENDIF

      RETURN
      END

