
      SUBROUTINE trrnge(ifile, rnglw, rnghi, qextrm, qintgr,
     &                  irnglw, irnghi, qvalid)

      INCLUDE '../../inc/xspec.inc'

      REAL  rnglw, rnghi

      INTEGER ifile, irnglw, irnghi

      LOGICAL qextrm(2), qintgr(2)
      LOGICAL qvalid

c Subroutine to convert ranges given in energies or wavelengths to 
c those given in channels. Also catches cases of the range set to 
c the extremum.

c Arguments :
c    ifile      i       i: Dataset number
c    rnglw      r       i: Low end of range entered
c    rnghi      r       i: High end of range entered
c    qextrm     l       i: True if range is set to extremum
c    qintgr     l       i: True if integer entered
c    irnglw     i       r: Low end of range
c    irnghi     i       r: High end of range
c    qvalid     l       r: If false then the specified energy/wavelength range 
c                          encompasses no bins

      REAL keVtoA
      PARAMETER (keVtoA=12.398419)

      REAL rlow, rhigh
      INTEGER i

      LOGICAL qup, qenerg
      LOGICAL qext(2), qint(2)

      REAL    rgerng
      INTEGER dgchan, rgnenr
      CHARACTER xpgxop*4
      EXTERNAL dgchan, rgerng, xpgxop, rgnenr

c  First check whether the information is available to do the translation.

      IF ( .NOT.qintgr(1) .AND. .NOT.qintgr(2) .AND. 
     &     rgnenr(ifile) .EQ. 0 )THEN
         CALL xwrite(
     & 'Cannot ignore energy range when no energy bins are defined.', 5)
         qvalid = .FALSE.
         RETURN
      ENDIF
      qvalid = .TRUE.

c  The QUP logical specifies whether the channels are stored in increasing
c  order of energy. Note that if no response is available we assume that
c  channels are increasing order so ** works correctly.

      qup = .TRUE.
      IF ( rgnenr(ifile) .NE. 0 .AND. 
     &     ( rgerng(dgchan(ifile), ifile, 'h') .LT. 
     &       rgerng(1, ifile, 'h') ) ) THEN
         qup = .FALSE.
      ENDIF

c  The QENERG logical specifies whether the input has been given in energy
c  or wavelengths.

      IF ( xpgxop() .EQ. 'wave' ) THEN
         qenerg = .FALSE.
      ELSE
         qenerg = .TRUE.
      ENDIF

c  If we are in wavelength space and bounds are not channels then have to 
c  convert the lower and upper bounds into energy and swap them 

      IF ( .NOT.qenerg .AND. .NOT.(qintgr(1).AND.qintgr(2)) ) THEN

         qint(1) = qintgr(2)
         qint(2) = qintgr(1)
         qext(1) = qextrm(2)
         qext(2) = qextrm(1)

         IF ( qintgr(1) ) THEN
            rhigh = rnglw
         ELSE
            IF ( rnglw .NE. 0. ) THEN
               rhigh = keVtoA/rnglw
            ELSE
               qext(2) = .TRUE.
            ENDIF
         ENDIF

         IF ( qintgr(2) ) THEN
            rlow = rnghi
         ELSE
            IF ( rnghi .NE. 0. ) THEN
               rlow = keVtoA/rnghi
            ELSE
               qvalid = .FALSE.
               RETURN
            ENDIF
         ENDIF

      ELSE

         rlow = rnglw
         rhigh = rnghi
         qint(1) = qintgr(1)
         qint(2) = qintgr(2)
         qext(1) = qextrm(1)
         qext(2) = qextrm(2)

      ENDIF

c  Do the low energy end

      IF ( qext(1) ) THEN

c  The extreme end. This is the first channel if energy increasing and the
c  last channel if it is decreasing.

         IF ( qup ) THEN
            irnglw = 1
         ELSE
            irnglw = dgchan(ifile)
         ENDIF

c  If the second argument is an integer then this extreme always refers to
c  the first channel

         IF ( qint(2) ) irnglw = 1

      ELSE

         IF ( qint(1) ) THEN

            irnglw = NINT(rnglw)

         ELSE

c  The input was in terms of energy or wavelength so find the 
c  appropriate channel. First trap out the low end being larger
c  than the largest value stored.

            IF ( qup ) THEN
               IF ( rlow .GT. rgerng(dgchan(ifile), ifile, 'h') ) THEN
                  qvalid = .FALSE.
               ENDIF
            ELSE
               IF ( rlow .GT. rgerng(1, ifile, 'l') ) THEN
                  qvalid = .FALSE.
               ENDIF
            ENDIF
            IF ( .NOT.qvalid ) RETURN

c  If the channels are stored in increasing energy then start at first
c  channel and run up till the requested energy exceeds a lower energy
c  range

            IF ( qup ) THEN

               i = 1
               DO WHILE ( i .LE. dgchan(ifile) .AND. 
     &                    rlow .GT. rgerng(i, ifile, 'l') )
                  i = i + 1
               ENDDO
               IF ( i .GT. 1 ) i = i - 1
               irnglw = i

            ELSE

c  If the channels are stored in decreasing energy then start at last
c  channel and run down till the requested energy is above an upper energy
c  range

               i = dgchan(ifile)
               DO WHILE ( i .GE. 1 .AND. 
     &                    rlow .GT. rgerng(i, ifile, 'h') )
                  i = i - 1
               ENDDO
               IF ( i .LT. dgchan(ifile) ) i = i + 1
               irnglw = i

            ENDIF

         ENDIF

      ENDIF

c  Now the high end 

      IF ( qext(2) ) THEN

c  The extreme end. This is the last channel if energy is stored in increasing
c  order otherwise it is the first channel.

         IF ( qup ) THEN
            irnghi = dgchan(ifile)
         ELSE
            irnghi = 1
         ENDIF

c  If the first argument is an integer then this extreme always refers to
c  the last channel

         IF ( qint(1) ) irnghi = dgchan(ifile)

      ELSE

         IF ( qint(2) ) THEN

            irnghi = NINT(rnghi)

         ELSE

c  The input was in terms of energy or wavelength so find the 
c  appropriate channel. First trap out the high end being smaller
c  than the smallest value stored.

            IF ( qup ) THEN
               IF ( rhigh .LT. rgerng(1, ifile, 'l') ) THEN
                  qvalid = .FALSE.
               ENDIF
            ELSE
               IF ( rhigh .LT. rgerng(dgchan(ifile), ifile, 'h') ) THEN
                  qvalid = .FALSE.
               ENDIF
            ENDIF
            IF ( .NOT.qvalid ) RETURN


c  If the channels are stored in increasing energy then start at last
c  channel and run down till the requested energy is less than a lower 
c  energy range

            IF ( qup ) THEN

               i = dgchan(ifile)
               DO WHILE ( i .GE. 1 
     &             .AND. rhigh .LT. rgerng(i, ifile, 'h') )
                  i = i - 1
               ENDDO
               IF ( i .LT. dgchan(ifile) ) i = i + 1
               irnghi = i

            ELSE

c  If the channels are stored in decreasing energy then start at first
c  channel and run up till the requested energy is less than a lower 
c  energy range

               i = 1
               DO WHILE ( i .LE. dgchan(ifile) .AND. 
     &                    rhigh .LT. rgerng(i, ifile, 'l') )
                  i = i + 1
               ENDDO
               IF ( i .GT. 1 ) i = i - 1
               irnghi = i

            ENDIF

         ENDIF

      ENDIF

c  In the case that the user specified the range backwards do a swap

      IF ( irnghi .LT. irnglw ) THEN
         i = irnghi
         irnghi = irnglw
         irnglw = i
      ENDIF

      RETURN
      END

