
      SUBROUTINE ldparr(imodval, fluxar, ear, ixopt, y, npts, cmd, 
     &                  ncmd, qcount)

      INTEGER imodval, npts, ncmd, ixopt
      REAL y(*)
      REAL fluxar(*), ear(0:*)

      LOGICAL qcount
      CHARACTER cmd(*)*(*)


c  Subroutine to load the main array for plots based on the data bins.
c  kaa  Dec 23, 1991.

c  Arguments
c imodval   r    		i: pointer to folded model values
c fluxar    r    		i: model fluxes in energy space
c ear       r    		i: energies for model fluxes
c ndtrgp    i    		i: number of data groups
c ixopt     i    		i: x-axis option - 1 channels
c                                                  2 energy
c                                                  3 wavelength
c y	    r			r: 2-d data array - 1&2  x-axis range
c						    3&4  data and error
c						    5    folded model
c						    6    model
c npts	    i			r: number of points
c cmd	    c*(*)		i/r: PLT commands
c ncmd      i			i/r: number of PLT commands
c qcount    l			i:plot in counts rather than counts/sec.

      INCLUDE '../../inc/xspec.inc'

      REAL NO, keVtoA
      PARAMETER (NO=-1.2e-34, keVtoA=12.398419)

      INTEGER ibin(MAXFIL)

      REAL    data, error, fmod, chemax, chemin, flux, eflux
      INTEGER ichtot, mxgmph, igroup, idset
      INTEGER icbeg, icsr, icsp, nbinrt, icb, ice
      INTEGER ielt, i, j, k, ioff, ntot, ierr
      LOGICAL qsingl, qdone

C  pointer for temporary

      INTEGER tpt

      INTEGER dgndst, dgnplg, rgnenr, rgdtse, rgbear, dgnchn, cgncmp
      INTEGER dgdpnt
      EXTERNAL dgndst, dgnplg, rgnenr, rgdtse, rgbear, dgnchn, cgncmp
      EXTERNAL dgdpnt

C  Get the memory for the temporary array - the maximum size that is
C  needed is the total number of channels (given by dgnchn()) times the
C  number of attributes for each channel

      tpt = -1
      ntot = 6*(dgnchn()+dgndst())
      CALL udmget(ntot, 6, tpt, ierr)
      IF ( ierr .NE. 0 ) THEN
         CALL xwrite(' Memory allocation failed in LDPARR', 2)
         RETURN
      ENDIF

c  Turn skip on to separate plot groups

      cmd(ncmd+1) = 'skip on'
      ncmd = ncmd + 1

c  loop round the plot groups accumulating the array

      DO idset = 1, dgndst()
         ibin(idset) = 0
      ENDDO

      npts = 0
      DO igroup = 1, dgnplg()
 
         icbeg = 0

c  Get the plot data info for this plot group

         CALL gtdpin(igroup, ichtot, mxgmph, qsingl, idset)

         icsr = 1
         icsp = mxgmph
         qdone = .false.

c  Get the first data point for this group

         CALL gtplin(igroup, icsr, icsp, ibin, MEMR(imodval), 
     &               MEMR(dgdpnt('src')), MEMR(dgdpnt('csv')), 
     &               MEMR(dgdpnt('are')),
     &               MEMR(dgdpnt('cor')), MEMR(dgdpnt('bkg')), 
     &               MEMR(dgdpnt('cbv')), data, fmod, error, chemin,
     &               chemax, icb, ice, nbinrt, ixopt, qcount)
         IF ( nbinrt .LE. 0 ) qdone = .TRUE.

c  Loop over the points to be plotted

         DO WHILE ( .NOT.qdone )

            npts = npts + 1
            ioff = (npts-1)*6 + 1
            IF (qsingl) qdone = .true.

c  Store the data, error and folded model

            MEMR(tpt+ioff+2) = data
            MEMR(tpt+ioff+3) = error
            MEMR(tpt+ioff+4) = fmod

c  If required, integrate the model over this energy range

            MEMR(tpt+ioff+5) = 0.
            IF ( rgnenr(idset) .GT. 0 .AND. cgncmp() .GT. 0 ) THEN
               ielt = rgdtse(idset) + 1
               CALL intmod(rgnenr(idset), fluxar(ielt), 
     &                     ear(rgbear(idset)), chemin, chemax, flux, 
     &                     eflux)
               IF ( ixopt .EQ. 1 .OR. ixopt .EQ. 2 ) THEN
                  IF ( (chemax-chemin) .GT. 0 ) THEN
                     MEMR(tpt+ioff+5) = flux/(chemax-chemin)
                  ENDIF
               ELSEIF ( ixopt .EQ. 3 ) THEN
                  IF ( chemax .GT. 0. .AND. chemin .GT. 0. ) THEN
                     MEMR(tpt+ioff+5) = flux/( (keVtoA/chemin)
     &                                        -(keVtoA/chemax) )
                  ENDIF
               ENDIF
            ENDIF

c  Set the x-axis

            IF ( ixopt .EQ. 1 ) THEN
               IF (qsingl) THEN
                  MEMR(tpt+ioff) = float(icbeg) - 0.5 + ichtot/2.
                  MEMR(tpt+ioff+1) = ichtot/2.
               ELSE
                  MEMR(tpt+ioff) = icbeg - 0.5 + float(ice+1+icb)/2.
                  MEMR(tpt+ioff+1) = float(ice+1-icb)/2.
               ENDIF
            ELSEIF ( ixopt .EQ. 2 ) THEN
               MEMR(tpt+ioff) = (chemax+chemin)/2.
               MEMR(tpt+ioff+1) = (chemax-chemin)/2.
            ELSEIF ( ixopt .EQ. 3 ) THEN
               IF ( chemax .LE. 0. OR. chemin .LE. 0. ) THEN
                  MEMR(tpt+ioff)  = NO
                  MEMR(tpt+ioff+1)= NO
               ELSE
                  MEMR(tpt+ioff)  =((keVtoA/chemax)+(keVtoA/chemin))/2.
                  MEMR(tpt+ioff+1)=((keVtoA/chemin)-(keVtoA/chemax))/2.
               ENDIF
            ENDIF

c  Get next plot data channel

            icsr = ice + 1
            IF(.NOT.qdone) THEN
               CALL gtplin(igroup, icsr, icsp, ibin, MEMR(imodval), 
     &                     MEMR(dgdpnt('src')), MEMR(dgdpnt('csv')),
     &                     MEMR(dgdpnt('are')),
     &                     MEMR(dgdpnt('cor')), MEMR(dgdpnt('bkg')), 
     &                     MEMR(dgdpnt('cbv')), data, fmod,
     &                     error, chemin, chemax, icb, ice, nbinrt, 
     &                     ixopt, qcount)
               IF ( nbinrt .LE. 0 ) qdone = .TRUE.

            ENDIF

         ENDDO

         icbeg = icbeg + ichtot

c Write the skip line between plot groups

         IF ( igroup .NE. dgnplg() ) THEN
            npts = npts + 1
            ioff = (npts-1)*6 + 1
            DO j = 0, 5
               MEMR(tpt+ioff+j) = NO
            ENDDO
         ENDIF

c  End of loop over plot groups

      ENDDO

C  Now load the contents of the temporary array into the Y array. The Y
C  array requires each vector be a continuous sequence.

      DO k = 1, 6

         ioff = (k-1) * npts
         DO i = 1, npts

            y(i + ioff) = MEMR(tpt + (i-1)*6 + k)

         ENDDO

      ENDDO

C  and free up the memory used by the temporary file

      CALL udmfre(tpt, 6, ierr)
      IF ( ierr .NE. 0 ) 
     &     CALL xwrite(' Memory deallocation failed in LDPARR', 2)

      RETURN
      END



